from django.db import models
from django import forms
from django.utils import simplejson
 
class JsonWidget(forms.Textarea):
    def render(self, name, value, attrs=None):
        if value is None:
            value = u""
        if not isinstance(value, basestring):
            value = simplejson.dumps(value, indent=2)
        return super(JsonWidget, self).render(name, value, attrs)


class JsonFormField(forms.CharField):
    def __init__(self, *args, **kwargs):
        kwargs['widget'] = JsonWidget
        super(JsonFormField, self).__init__(*args, **kwargs)
 
    def clean(self, value):
        if not value: 
            return None
        try:
            return simplejson.loads(value)
        except Exception, exc:
            raise forms.ValidationError(u'JSON decode error: %s' % (unicode(exc),))

 
class JsonField(models.TextField):
    """
    A django model field used to store data in json format.
    """
    __metaclass__ = models.SubfieldBase
 
    def formfield(self, **kwargs):
        return super(JsonField, self).formfield(form_class=JsonFormField, **kwargs)
        
    def to_python(self, value):
        if value is None:
            return None
        try:
            if isinstance(value, basestring):
                return simplejson.loads(value)
        except ValueError:
            return None
        return value
 
    def get_db_prep_save(self, value, connection=None):
        if value is None:
            return None if self.null else u""
        return simplejson.dumps(value)
         
    def value_to_string(self, obj):
        value = self._get_val_from_obj(obj)
        return self.get_db_prep_value(value)
