import sys
from optparse import make_option

from django.core.management.base import BaseCommand
from django.contrib.contenttypes.models import ContentType

from popularity import settings, models, shortcuts

class Command(BaseCommand):
    """
    Reset the rank for objects, creating a *Rank* record for unranked objects.
    """
    option_list = BaseCommand.option_list + (
        make_option('-m', "--method", 
            action='store', dest='method', default="unranked", type='string',
            help=('Reset popularity for "unranked" (default) or for "all" objects.')
        ),
        make_option('-l', "--limit", 
            action='store', dest='limit', default=settings.CAP, type='int',
            help=('The total number of objects to rank (default=%d)' % settings.CAP)
        ),
    )
    help = 'Reset the rank for objects, creating a *Rank* record for unranked objects.'
        
    def handle(self, **options):
        method = options["method"]
        limit = options["limit"]
        if method not in ("unranked", "all"):
            print "Error: invalid method %s." % method
            sys.exit(1)
        for model_name in settings.RANKABLE_MODELS:
            print "Populating %s" % model_name
            queryset = shortcuts.get_queryset(model_name, use_patch=False).order_by("-id")[:limit]
            content_type = ContentType.objects.get_for_model(queryset.model)
            counter = 0
            for obj in queryset:
                rank, created = models.Rank.objects.get_or_create(
                    content_type=content_type, object_id=obj.pk)
                if created or (method == "all"):
                    rank.update_history(rank.weekly)
                    rank.save()
                    counter += 1
            print "- %d objects ranked." % counter
