from django.contrib.contenttypes.models import ContentType

from popularity import utils

# preload default backend and handler
backend = utils.get_backend()
handler = utils.get_handler()

def hit(obj, points=1, request=None):
    """
    Shortcut for hitting default backend using default handler.
    """
    handler(backend, request, obj, points)
    
def patch(queryset):
    """
    Patch a queryset that you want to order by rank.
    """
    content_type = ContentType.objects.get_for_model(queryset.model)
    return queryset.filter(ranks__content_type=content_type.pk)
    
def get_queryset(data, use_patch=True):
    """
    Return a queryset of active published objects related to *data*.
    
    The passed argument *data* can be:
        - a model name (e.g. 'auth.user')
        - a model class (e.g. <User Model>)
        - a content type class (e.g. <ContentType Model>)
    
    This shortcut uses a callable to get active published objects.
    The callable can be customized by the user using 
    *settings.POPULARITY_QUERYSET_CALLBACK*. The callable must accept 
    a model argument and the patch function and must return a 
    queryset of objects of that model (better if patched).
    """
    patch_function = patch if use_patch else lambda queryset: queryset
    if isinstance(data, basestring):
        from django.db.models import get_model
        model = get_model(*data.split("."))
    else:
        try:
            model = data.model_class()
        except AttributeError:
            model = data
    return utils.queryset_callback(model, patch_function)
